#Copyright Massachusetts Institute of technology, 1998.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

use strict;
require Exporter;
require AutoLoader;

use W3C::SAX::XmlElementTree;
use W3C::SAX::SAXParseException;
use W3C::SAX::AttributeListImpl;

$W3C::SAX::AttributeListImpl::REVISION = '$Id: AttributeListImpl.pm,v 1.15 1999/12/14 13:14:41 eric Exp $ ';

package W3C::SAX::AttributeListImpl;

use vars qw($VERSION $DSLI @ISA @EXPORT @EXPORT_OK @TODO);
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = 0.96;
$DSLI = 'adpO';
@TODO = ('It is impossible to to distinguish getValue("1") from getValue(1) so attributes with a name that matches m/\A\d+\Z/ will be lost unless retrieved by number.');

#####
# per-class data

my $Debugging = 0;	# whether to show debugging stuff
my $Outstanding = 0;	# count of outstanding connections
*W3C::SAX::AttributeListImpl::NAME = \0;
*W3C::SAX::AttributeListImpl::TYPE = \1;
*W3C::SAX::AttributeListImpl::VALUE = \2;

#####
# per-object data
# ATTIBUTES	- array of all the collected attributes
# DEBUG		- per-object control of debugging info

#####
# new - prepare a W3C::SAX::AttributeListImpl with a new connection

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    $self->{DEBUG} = 0;
    # "private" data
    $self->{"_OUTSTANDING"} = \$Outstanding;
    $self->{ATTRIBUTES} = [];
    bless ($self, $class);
    $self->{DEFAULT_NAMESPACE} = shift; # @@@ xmlns
    my $atts = shift;
    $self->setAttributeList($atts) if (defined $atts);
    ++ ${ $self->{"_OUTSTANDING"} };
    return $self;
}

#####
# setAttributeList - replace into attribute list

sub setAttributeList {
    my $self = shift;
    my $atts = shift;
    my $size = $atts->getLength();
    $self->clear();
    for (my $i = 0; $i < $size; $i++) {
	$self->addAtribute($atts->getName($i), $atts.getType($i), $atts->getValue($i));
    }
}

#####
# addAttribute - add new attribute to attribute list

sub addAttribute {
    my $self = shift;
    my $name = shift;
    my $type = shift;
    my $value = shift;
#    $name = $self->{DEFAULT_NAMESPACE}.':'.$name if ($name !~ m/\A [^:]+ : /x); # xmlns
    push (@{$self->{ATTRIBUTES}}, [$name, $type, $value]);
}

#####
# removeAttribute - and get rid of it

sub removeAttribute {
    my ($self, $name) = @_;

    my $new = {};
    my $attrs = $self->{ATTRIBUTES};
    for (my $i = 0; $i <= $#$attrs; $i++) {
#	push (@$new, $$attrs[$i]) if ($ {@$attrs[$i]}[$W3C::SAX::AttributeListImpl::NAME] ne $name);
	if ($attrs->[$i][$W3C::SAX::AttributeListImpl::NAME] eq $name) {
	    splice (@$attrs, $i, 1);
#	    return;
	}
    }
#    $self->{ATTRIBUTES} = $new;   
}

#####
# clear - get rid of all attributes

sub clear {
    my $self = shift;
    my $name = shift;
    $self->{ATTRIBUTES} = [];
}

#####
# getLength - return the number of attribute

sub getLength {
    my $self = shift;
    return $#{$self->{ATTRIBUTES}} + 1;
}

#####
# getName - return the name of the i'th attribute

sub getName {
    my $self = shift;
    my $i = shift;
    return undef if ($i > $#{$self->{ATTRIBUTES}});
    return @{@{$self->{ATTRIBUTES}}[$i]}[$W3C::SAX::AttributeListImpl::NAME];
}

#####
# getType - return the type of the i'th attribute

sub getType {
    my $self = shift;
    my $i = shift;
    return $self->getOne($i, $W3C::SAX::AttributeListImpl::TYPE);
}

#####
# getValue - return the value of the i'th attribute

sub getValue {
    my $self = shift;
    my $i = shift;
    return $self->getOne($i, $W3C::SAX::AttributeListImpl::VALUE);
}

#####
# getOne - return the n'th of the i'th attribute

sub getOne {
    my $self = shift;
    my $arg1 = shift;
    my $n = shift;
    my $i;
    if ($arg1 =~ m/\A\d+\Z/) {
	$i = $arg1;
    } else {
	for ($i = 0; $i <= $#{$self->{ATTRIBUTES}}; $i++) {
	    last if ($self->{ATTRIBUTES}->[$i]->[$W3C::SAX::AttributeListImpl::NAME] eq $arg1);
	}
	# if not found, $i > $#$attrs so return undef
    }
    return undef if ($i > $#{$self->{ATTRIBUTES}});
    return $self->{ATTRIBUTES}->[$i]->[$W3C::SAX::AttributeListImpl::VALUE];
}

#####
# sum - compute a 16 bit checksum for all except the listed attributes

sub sum {
    my ($self, $exclude) = @_;
    my $sum = 0;
    foreach my $attribute (@{$self->{ATTRIBUTES}}) {
	$sum ^= unpack ('%16C*', $attribute->[1]) if (!grep {$_ eq $attribute->[0]} @$exclude);
    }
    return $sum;
}

#####
# show - dump the attribute list

sub show {
    my ($self, $prefix, $namespaceHandler) = @_;
    my $ret; # = 'default namespace: '.$self->{DEFAULT_NAMESPACE}."\n";
    map {
	my $name = $_->[$W3C::SAX::AttributeListImpl::NAME];
	$name = $namespaceHandler->unmapNamespace($name) if ($namespaceHandler);
	$ret .=$prefix.$name.'('.$_->[$W3C::SAX::AttributeListImpl::TYPE].'): '.$_->[$W3C::SAX::AttributeListImpl::VALUE]."\n"
	} @{$self->{ATTRIBUTES}};
    return $ret;
}

#####
# debug - cannonical debugging stuff from
# http://www.perl.com/CPAN-local/doc/manual/html/pod/perltoot/Debuging_Methods.html

sub debug {
    my $self = shift;
    warn "usage: thing->debug(level)"    unless @_ == 1;
    my $level = shift;
    if (ref($self))  {
	$self->{"_DEBUG"} = $level;         # just myself
    } else {
	$Debugging        = $level;         # whole class
    }
}

#####
# DESTROY - hook to display debugging info

sub DESTROY {
    my $self = shift;
    if ($Debugging || $self->{"_DEBUG"}) {
	warn "W3C::SAX::AttributeListImpl destroying $self " . $self->name;
    }
    -- ${ $self->{"_OUTSTANDING"} };
}

#####
# END - hook to display debugging info

sub END {
    if ($Debugging) {
	print "All AttributeListImpls are going away now.\n";
    }
}

1;

__END__

=head1 NAME

W3C::SAX::AttributeListImpl - implementation of the SAX AttributeList interface

=head1 SYNOPSIS

  use W3C::SAX::AttributeListImpl;
  my $attributeList = new W3C::SAX::AttributeListImpl;
  $attributeList->addAttribute($attrname, "CDATA", $attrvalue);

=head1 DESCRIPTION

This is an implementation of the SAX AttributeList interface.

This module is part of the W3C::SAX CPAN module.

=head1 AUTHOR

Eric Prud\'hommeaux <eric@w3.org>

=head1 SEE ALSO

W3C::SAX::XmlParser(3) perl(1).

=cut

