#Copyright Massachusetts Institute of technology, 1998.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

use strict;
require Exporter;
require AutoLoader;

use W3C::SAX::SAXParseException;

$W3C::SAX::ErrorHandlerImpl::REVISION = '$Id: ErrorHandlerImpl.pm,v 1.5 1999/12/14 13:14:41 eric Exp $ ';

package W3C::SAX::ErrorHandlerImpl;
use W3C::Util::Exception;
use vars qw($VERSION $DSLI @ISA @EXPORT @EXPORT_OK @TODO);
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = 0.91;
$DSLI = 'adpO';
@TODO = ();

# this error handler is general enough to handle SAXExceptions and RdfDBExceptions

sub new {
    my ($class, $parms) = @_;
    my $proto = ref $class || $class;
    my $self = $parms ? $parms : {};
    bless($self, $proto);
    return $self;
}

sub setDocumentLocator {
    my ($self, $documentLocator) = @_;
    $self->{-documentLocator} = $documentLocator;
}

sub makeString {
    my ($self, $exception) = @_;
    my $ret = $exception->toString;
    if ($exception->isa('W3C::Rdf::RdfDBException')) {
	$ret .= ' at '.$self->{-documentLocator}->getPublicId.':'.
	    $self->{-documentLocator}->getLineNumber.'.'.
		$self->{-documentLocator}->getColumnNumber;
    }
    $DB::single = 1;
    return $ret;
}

sub warning {
    my ($self, $exception) = @_;
    print 'warning: '.$self->makeString($exception)."\n";
}

sub error {
    my ($self, $exception) = @_;
    warn 'error: '.$self->makeString($exception)."\n";
}

sub fatalError {
    my ($self, $exception) = @_;
    &throw($exception);
}

1;

__END__

=head1 NAME

W3C::SAX::ErrorHandlerImpl - implementation of the SAX ErrorHandler interface

=head1 SYNOPSIS

  my $xmlParser = new W3C::SAX::PerlXmlParser;
  use W3C::SAX::ErrorHandlerImpl;
  my $errorHandler = new W3C::SAX::ErrorHandlerImpl({-documentLocator => $xmlParser});
  $xmlParser->setErrorHandler($errorHandler);

=head1 DESCRIPTION

This is an implementation of the SAX ErrorHandler interface.

This module is part of the W3C::SAX CPAN module.

=head1 AUTHOR

Eric Prud\'hommeaux <eric@w3.org>

=head1 SEE ALSO

W3C::SAX::XmlParser(3) W3C::SAX::SAXParseException(3) perl(1).

=cut

