#Copyright Massachusetts Institute of technology, 1998.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

use strict;
require Exporter;
require AutoLoader;

$W3C::SAX::HandlerBase::REVISION = '$Id: HandlerBase.pm,v 1.19 1999/12/14 13:14:41 eric Exp $ ';

package W3C::SAX::HandlerBase;
use W3C::Util::Exception;
use vars qw($VERSION $DSLI @ISA @EXPORT @EXPORT_OK @CHANGES);
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = 0.97;
$DSLI = 'adpO';
@CHANGES = (
	    {'0.96' => ['added NamespaceHandler::addNamespace']}, 
	    {'0.95' => ['started keeping track']});

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    bless ($self, $class);
    return $self;
}

#####
# entityResolver routines

sub resolveEntity {
    my $self = shift;
    my $publicId = shift;
    my $systemId = shift;
    return undef;
}

#####
# DTDHandler routines

sub notationDecl {
    my $self = shift;
    my $name = shift;
    my $publicId = shift;
    my $systemId = shift;
}

sub unparsedEntityDecl {
    my $self = shift;
    my $name = shift;
    my $publicId = shift;
    my $systemId = shift;
    my $notationName = shift;
}

#####
# documentHandler routines

sub setDocumentLocator {
    my $self = shift;
}

sub startDocument {
    my $self = shift;
}

sub endDocument {
    my $self = shift;
}

sub startElement {
    my $self = shift;
    my $name = shift;
    my $attributeList = shift;
}

sub endElement {
    my $self = shift;
    my $name = shift;
}

sub characters {
    my $self = shift;
    my $ch = shift;
    my $start = shift;
    my $length = shift;
}

sub ignorableWhitespace {
    my $self = shift;
    my $ch = shift;
    my $start = shift;
    my $length = shift;
}

sub processingInstruction {
    my $self = shift;
    my $target = shift;
    my $data = shift;
}

#####
# ErrorHandler routines

sub warning {
    my $self = shift;
    my $exception = shift;
}

sub error {
    my $self = shift;
    my $exception = shift;
}

sub fatalError {
    my $self = shift;
    my $exception = shift;
    &throw($exception);
}

package W3C::SAX::HandlerStream;
use W3C::Util::Exception;
@W3C::SAX::HandlerStream::ISA = ("W3C::SAX::HandlerBase");

#####
# new - prepare a W3C::Rdf::RdfParser with a new connection

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new();
    $self->{DOC_HANDLER} = shift;
    $self->{ERROR_HANDLER} = shift;
    $self->{ENT_HANDLER} = shift;
    $self->{DTD_HANDLER} = shift;
    return $self;
}

sub setDocumentHandler {
    my ($self, $handler) = @_;
    $self->{DOC_HANDLER} = $handler;
}
sub setErrorHandler {
    my ($self, $handler) = @_;
    $self->{ERROR_HANDLER} = $handler;
}
sub setEntityResolver {
    my ($self, $handler) = @_;
    $self->{ENT_HANDLER} = $handler;
}
sub setDTDHandler {
    my ($self, $handler) = @_;
    $self->{DTD_HANDLER} = $handler;
}

#####
# entityResolver routines

sub resolveEntity {
    my $self = shift;
    return $self->{ENT_HANDLER}->resolveEntity(@_);
}

#####
# DTDHandler routines

sub notationDecl {
    my $self = shift;
    return $self->{DOC_HANDLER}->notationDecl(@_);
}

sub unparsedEntityDecl {
    my $self = shift;
    $self->{DOC_HANDLER}->unparsedEntityDecl(@_);
}

#####
# documentHandler routines

sub setDocumentLocator {
    my $self = shift;
    $self->{DOC_HANDLER}->setDocumentLocator(@_);
}

sub startDocument {
    my $self = shift;
    $self->{DOC_HANDLER}->startDocument(@_);
}

sub endDocument {
    my $self = shift;
    $self->{DOC_HANDLER}->endDocument(@_);
}

sub startElement {
    my $self = shift;
    $self->{DOC_HANDLER}->startElement(@_);
}

sub endElement {
    my $self = shift;
    $self->{DOC_HANDLER}->endElement(@_);
}

sub characters {
    my $self = shift;
    $self->{DOC_HANDLER}->characters(@_);
}

sub ignorableWhitespace {
    my $self = shift;
    $self->{DOC_HANDLER}->ignorableWhitespace(@_);
}

sub processingInstruction {
    my $self = shift;
    $self->{DOC_HANDLER}->processingInstruction(@_);
}

sub addNamespace {
    my ($self) = shift;
    $self->{DOC_HANDLER}->addNamespace(@_);
}

#####
# ErrorHandler routines

sub warning {
    my $self = shift;
    $self->{ERROR_HANDLER}->warning(@_);
}

sub error {
    my $self = shift;
    $self->{ERROR_HANDLER}->error(@_);
}

sub fatalError {
    my $self = shift;
    $self->{ERROR_HANDLER}->fatalError(@_);
    my $exception = shift;
    &throw($exception);
}

sub show {
    my ($self) = shift;
    my $ret;
    eval {$ret = $self->{DOC_HANDLER}->show(@_)."\n";}; # don't assume the function exists
    return $ret;
}

# W3C::SAX::NamespaceHandler is a simple stream to implement hierarchical namespaces
# according to http://www.w3.org/TR/1999/REC-xml-names-19990114. It maintains a
# NAMESPACE_STACK so namespace mappings fall out of scope when their bounding
# element ends (see http://www.w3.org/TR/1999/REC-xml-names-19990114#scoping).

# This also stores the namespace mappings by their source ID after they have
# gone out of scope. This makes it possible to call unmapNamespace after the
# parser has finished with the document.

package W3C::SAX::NamespaceHandler;
@W3C::SAX::NamespaceHandler::ISA = ("W3C::SAX::HandlerStream");

sub new {
    require W3C::SAX::AttributeListImpl;
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new(@_);
    bless ($self, $class);
    $self->{NAMESPACE_STACK} = [[undef, {}]]; # default namespace and top namespace frame
    $self->{NAMESPACE_REVERSE} = {};
    $self->{UNKNOWN_NAMESPACES} = [];
    return $self;
}

sub setDocumentLocator {
    my ($self, $locator) = @_;
    $self->SUPER::setDocumentLocator($locator);
    $self->{SOURCE_ID} = $locator->getPublicId;
}

#    $namespaceHandler->addNamespace('xsl', 'http://www.w3.org/TR/WD-xsl', $inputSource->getPublicId);
sub addNamespace {
    my ($self, $prefix, $expanded, $sourceId) = @_;
    my $namespaceFrame = $self->{NAMESPACE_STACK}[-1][1];
    $namespaceFrame->{$prefix} = $expanded;
    $self->{NAMESPACE_REVERSE}{$sourceId}{$expanded} = $prefix
	if (!exists $self->{NAMESPACE_REVERSE}{$sourceId}{$expanded});
}

sub startElement {
    my ($self, $name, $attributeList) = @_;
    my $sourceId = $self->{SOURCE_ID};
    my %attrs;

    # default namespace for this element is the namespace for the tag
    my ($defaultNamespace) = $name =~ m/\A (?: ([^:]* ) :)? /x;

    # inherit namespace from the parent element
#    my $namespaceFrame = $self->{NAMESPACE_STACK}[-1] ? {%{$self->{NAMESPACE_STACK}[-1][1]}} : {};
#    my $namespaceFrame = {%{$self->{NAMESPACE_STACK}[-1][1]}};

    # current mapping context now reflects these xmlns directives
#    push (@{$self->{NAMESPACE_STACK}}, [$defaultNamespace, $namespaceFrame]);
    # create current context from the default namespace and a copy of the last namespace frame
    push (@{$self->{NAMESPACE_STACK}}, [$defaultNamespace, {%{$self->{NAMESPACE_STACK}[-1][1]}}]);

    # update namespace to reflect xmlns directives in the $attributeList
    for (my $i = 0; $i < $attributeList->getLength; $i++) {
	my ($attributeName, $attributeValue) = ($attributeList->getName($i), $attributeList->getValue($i));
	if ($attributeName =~ m/\A xmlns(:(\w+))? \Z/x) {
	    $self->addNamespace($2, $attributeValue, $sourceId);
	}
    }
    for (my $i = 0; $i < $attributeList->getLength; $i++) {
	my ($attributeName, $attributeValue) = ($attributeList->getName($i), $attributeList->getValue($i));
	if ($attributeName =~ m/\A xmlns(:(\w+))? \Z/x) {
	} else {
	    if (substr ($attributeName, -1) eq ':') {	
		$attrs{substr ($attributeName, 0, -1)} = $self->mapNamespace($attributeValue);
	    } else {
		$attrs{$attributeName} = $attributeValue;
	    }
	}
    }

    # build new AttributeList that has the mapped attribute names
    # my $mappedAttributes = new W3C::SAX::AttributeListImpl($defaultNamespace); # Why the hell does this give:
    # Undefined subroutine &W3C::SAX::NamespaceHandler::W3C::SAX::AttributeListImpl called at ../W3C/SAX/W3C::SAX::HandlerBase.pm line 297.
    $attributeList->clear;
    map {$attributeList->addAttribute($self->mapNamespace($_), "CDATA", $attrs{$_})} keys %attrs;

    # pass to regularly scheduled document handler
    $self->{DOC_HANDLER}->startElement($self->mapNamespace($name), $attributeList);
}

sub endElement {
    my ($self, $name) = @_;
    $self->{DOC_HANDLER}->endElement($self->mapNamespace($name));
    pop @{$self->{NAMESPACE_STACK}};
}

sub mapNamespace {
    my ($self, $toMap) = @_;
    my ($defaultNamespace, $namespaceFrame) = (@{$self->{NAMESPACE_STACK}[-1]});
    my ($namespace, $property) = $toMap =~ m/(?:([^:]*):)?(.*)/;
    $namespace ||= $defaultNamespace;
    if (exists $namespaceFrame->{$namespace}) {
	return $namespaceFrame->{$namespace}.$property;
    } else {
	push (@{$self->{UNKNOWN_NAMESPACES}}, $namespace);
	return $namespace.':'.$property;
    }
}

sub getUnknownNamespaces {
    my ($self) = @_;
    return @{$self->{UNKNOWN_NAMESPACES}};
}

# makes an attempt at reversing namespaces for printing - will not follow hierarchy
# $sourceId is optional
sub unmapNamespace ($;$) {
    my ($self, $name, $sourceId) = @_;
    my @idList = defined $sourceId ? ($sourceId) : keys %{$self->{NAMESPACE_REVERSE}};
    foreach my $id (@idList) {
	# poke through the keys - starting with the longest so we get the best match
	foreach my $key (sort {length $b <=> length $a} keys %{$self->{NAMESPACE_REVERSE}{$id}}) {
	    return ($id, $key, $self->{NAMESPACE_REVERSE}{$id}{$key}.':'.$1) if ($name =~ m/\A \Q$key\E (.*) \Z/x);
	}
    }
    return (undef, undef, $name);
}

package W3C::SAX::NamespaceAndStringHandler;
@W3C::SAX::NamespaceAndStringHandler::ISA = ("W3C::SAX::NamespaceHandler");

sub addStringMap ($$) {
    my ($self, $from, $to) = @_;
    $self->{STRING_REVERSE}{$from} = $to;
}

sub addResourceMap ($$) {
    my ($self, $resource) = @_;
    $resource =~ m/([^\\\/\:]+)$/ || m/([^\\\/\:]+)[^\\\/\:]$/;
    my $base = $1;
    my $to = $base; # start with the plain base name
    my $revNo = 0;
    while (exists $self->{STRING_FORWARD}{$to}) {$to = $base.'<'.++$revNo.'>'};
    $self->{STRING_FORWARD}{$to} = $resource;
    $self->{STRING_REVERSE}{$resource} = $to;
}

sub unmapString ($) {
    my ($self, $string) = @_;
    foreach my $key (sort {length $b <=> length $a} keys %{$self->{STRING_REVERSE}}) {
	return ($self->{STRING_REVERSE}{$key}.$1) if ($string =~ m/\A \Q$key\E (.*) \Z/x);
    }
    return $string;
}

sub unmapNamespace ($;$) {
    my ($self, $name, $sourceId) = @_;
    my ($id, $key, $translated) = $self->SUPER::unmapNamespace($name, $sourceId);
    $translated = $self->unmapString($translated) if (!defined $key);
    return ($id, $key, $translated);
}

package W3C::SAX::HandlerList;
use W3C::Util::Exception;
@W3C::SAX::HandlerList::ISA = ("W3C::SAX::HandlerBase");

#####
# new - prepare a W3C::Rdf::RdfParser with a new connection

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new();
    $self->{DOC_HANDLERS} = shift;
    $self->{ERROR_HANDLERS} = shift;
    $self->{ENT_HANDLERS} = shift;
    $self->{DTD_HANDLERS} = shift;
    return $self;
}

sub setDocumentHandler {
    my ($self, $handler) = @_;
    $self->{DOC_HANDLERS} = [$handler];
}
sub setErrorHandler {
    my ($self, $handler) = @_;
    $self->{ERROR_HANDLERS} = [$handler];
}
sub setEntityResolver {
    my ($self, $handler) = @_;
    $self->{ENT_HANDLERS} = [$handler];
}
sub setDTDHandler {
    my ($self, $handler) = @_;
    $self->{DTD_HANDLERS} = [$handler];
}

sub addDocumentHandler {
    my ($self, @handlers) = @_;
    push (@{$self->{DOC_HANDLERS}}, @handlers);
}
sub addErrorHandler {
    my ($self, @handlers) = @_;
    push (@{$self->{ERROR_HANDLERS}}, @handlers);
}
sub addEntityResolver {
    my ($self, @handlers) = @_;
    push (@{$self->{ENT_HANDLERS}}, @handlers);
}
sub addDTDHandler {
    my ($self, @handlers) = @_;
    push (@{$self->{DTD_HANDLERS}}, @handlers);
}

#####
# entityResolver routines

sub resolveEntity {
    my $self = shift;
    my $ret;
    map {return $ret if (defined ($ret = $_->resolveEntity(@_)))} @{$self->{ENT_HANDLERS}};
    return undef;
}

#####
# DTDHandler routines

sub notationDecl {
    my $self = shift;
    map {$_->notationDecl(@_)} @{$self->{DOC_HANDLERS}};
}

sub unparsedEntityDecl {
    my $self = shift;
    map {$_->unparsedEntityDecl(@_)} @{$self->{DOC_HANDLERS}};
}

#####
# documentHandler routines

sub setDocumentLocator {
    my $self = shift;
    map {$_->setDocumentLocator(@_)} @{$self->{DOC_HANDLERS}};
}

sub startDocument {
    my $self = shift;
    map {$_->startDocument(@_)} @{$self->{DOC_HANDLERS}};
}

sub endDocument {
    my $self = shift;
    map {$_->endDocument(@_)} @{$self->{DOC_HANDLERS}};
}

sub startElement {
    my $self = shift;
    map {$_->startElement(@_)} @{$self->{DOC_HANDLERS}};
}

sub endElement {
    my $self = shift;
    map {$_->endElement(@_)} @{$self->{DOC_HANDLERS}};
}

sub characters {
    my $self = shift;
    map {$_->characters(@_)} @{$self->{DOC_HANDLERS}};
}

sub ignorableWhitespace {
    my $self = shift;
    map {$_->ignorableWhitespace(@_)} @{$self->{DOC_HANDLERS}};
}

sub processingInstruction {
    my $self = shift;
    map {$_->processingInstruction(@_)} @{$self->{DOC_HANDLERS}};
}

sub addNamespace {
    my ($self) = shift;
    map {$_->addNamespace(@_)} @{$self->{DOC_HANDLERS}};
}

#####
# ErrorHandler routines

sub warning {
    my $self = shift;
    map {$_->warning(@_)} @{$self->{ERROR_HANDLERS}};
}

sub error {
    my $self = shift;
    map {$_->error(@_)} @{$self->{ERROR_HANDLERS}};
}

sub fatalError {
    my $self = shift;
    map {$_->fatalError(@_)} @{$self->{ERROR_HANDLERS}};
    my $exception = shift;
    &throw($exception->getMessage);
}

sub show {
    my ($self) = shift;
    my $ret;
    map {eval {$ret .= $_->show(@_)."\n";}} @{$self->{DOC_HANDLERS}}; # don't assume the function exists
    return $ret;
}

1;

package W3C::SAX::HandlerBase; # so make doesn't complain about the package

__END__

=head1 NAME

W3C::SAX::HandlerBase.pm - default methods for all the SAX handler routines

=head1 SYNOPSIS

  package myHandler;
  @myHandler::ISA = qw(W3C::SAX::HandlerBase);
  sub new {
    my $self = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new();
  }
  sub startElement {
  }
  sub endElement {
  }  

=head1 DESCRIPTION

HandlerBase provides default methods for all the SAX handler routines.
Other handy packages:
  HandlerStream - pass all handler calls to a downstream handler.
  HandlerList - pass all handler calls to a list of downstream handlers.
  NamespaceHandler - provide a hierarchical namespaces.

=head2 HandlerStream

  package myHandler;
  @myHandler::ISA = qw(W3C::SAX::HandlerStream);
  sub new {
    my ($self, $nextHandler) = @_;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new($nextHandler);
  }
  sub startElement {
    my ($self, $name, $attributeList) = @_;
    $self->SUPER::startElement($name, $attributeList);
  }
  sub endElement {
    my ($self, $name) = @_;
    $self->SUPER::endElement($name);
  }  

=head2 HandlerList

  use W3C::SAX::XmlParser;
  use W3C::SAX::VerboseXmlHandler;
  use W3C::SAX::InputSource;
  use W3C::SAX::XmlElementTree;
  my $xmlParser = new W3C::SAX::PerlXmlParser;
  my $verboseHandler = new W3C::SAX::VerboseXmlHandler;
  my $treeHandler = new W3C::SAX::XmlElementTree;

  my $handlerList = new W3C::SAX::HandlerList([$treeHandler, $verboseHandler], [$verboseHandler]);

  # set the W3C::SAX::HandlerList be the document handler and the error handler
  $xmlParser->setDocumentHandler($handler);
  $xmlParser->setErrorHandler($handler);

  # assign a URI to the XML in InputSorce and parse it
  my $inputSource = new W3C::SAX::FileInputSource($source);
  eval {$xmlParser->parse($inputSource);};

=head2 NamespaceHandler

  use W3C::SAX::XmlParser;
  use W3C::SAX::VerboseXmlHandler;
  use W3C::SAX::InputSource;
  use W3C::SAX::HandlerBase;
  my $xmlParser = new W3C::SAX::PerlXmlParser;
  my $verboseHandler = new W3C::SAX::VerboseXmlHandler;

  # use a W3C::SAX::NamespaceHandler stream to map from prefix:tag
  my $namespaceHandler = new W3C::SAX::NamespaceHandler($verboseHandler);

  # set the W3C::SAX::HandlerList be the document handler and the error handler
  $xmlParser->setDocumentHandler($namespaceHandler);
  $xmlParser->setErrorHandler($verboseHandler);

  # assign a URI to the XML in InputSorce and parse it
  my $inputSource = new W3C::SAX::FileInputSource($source);
  eval {$xmlParser->parse($inputSource);};

This module is part of the W3C::SAX CPAN module.

=head1 AUTHOR

Eric Prud\'hommeaux <eric@w3.org>

=head1 SEE ALSO

perl(1).

=cut
