#Copyright Massachusetts Institute of technology, 1998.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

use strict;
require Exporter;
require AutoLoader;

$W3C::SAX::InputSource::REVISION = '$Id: InputSource.pm,v 1.14 1999/12/14 13:14:41 eric Exp $ ';

use W3C::Util::Exception;

package W3C::SAX::InputSource;
use vars qw($VERSION $DSLI @ISA @EXPORT @EXPORT_OK @TODO);
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = 0.96;
$DSLI = 'adpO';
@TODO = ('this is radically different than the java equivilent. Not only is it not done, there are design conflicts between the Java and the Perl versions.');

#####
# per-class data

my $Debugging = 0;	# whether to show debugging stuff
my $Outstanding = 0;	# count of outstanding connections

#####
# per-object data
# ATTIBUTES	- array of all the collected attributes
# DEBUG		- per-object control of debugging info

#####
# new - prepare a W3C::SAX::InputSource with a new connection
#
# initializer - where to get data for byte stream
#	string - filename where XML is stored
#	SCALAR - ref to actual text of XML
#	ARRAY - array of strings of XML
#	HASH - dies - what could be done here?

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    $self->{DEBUG} = 0;
    # "private" data
    $self->{"_OUTSTANDING"} = \$Outstanding;
    my $initializer = shift;
    if (ref $initializer eq 'SCALAR') {
	$self->{BYTESTREAM} = $$initializer;
    } elsif(ref $initializer eq 'ARRAY') {
	$self->{BYTESTREAM} = join('', @$initializer);
    } elsif(ref $initializer eq 'HASH') {
	&throw(new W3C::Util::NotImplementedException({-message => "Can't initialize an W3C::SAX::InputSource from a hash.", 
						       -class => 'W3C::SAX::InputSource', -method => 'new'}));
    } else {
	local *HANDLE;
	open(HANDLE, $initializer) || &throw(new W3C::Util::FileNotFoundException({-filename => $initializer}));
	my $oldRS = $/;
	undef $/;
	$/ = $oldRS;
	my $toy = <HANDLE>;
	# @@@ there's probably a better way, but I'm sleepy
	my @tmp = <HANDLE>;
	$self->{BYTESTREAM} = join('', @tmp);
    }
    bless ($self, $class);
    ++ ${ $self->{"_OUTSTANDING"} };
    return $self;
}

#####
# setSytemId - 

sub setPublicId {
    my $self = shift;
    $self->{PUBLICID} = shift;
}

sub getPublicId {
    my $self = shift;
    return $self->{PUBLICID};
}

#####
# setSytemId - 

sub setSystemId {
    my $self = shift;
    $self->{SYSTEMID} = shift;
}

sub getSystemId {
    my $self = shift;
    return $self->{SYSTEMID};
}

#####
# setSytemId - 

sub setByteStream {
    my $self = shift;
    $self->{BYTESTREAM} = shift;
}

sub getByteStream {
    my $self = shift;
    return $self->{BYTESTREAM};
}

#####
# setSytemId - 

sub setEncoding {
    my $self = shift;
    $self->{ENCODING} = shift;
}

sub getEncoding {
    my $self = shift;
    return $self->{ENCODING};
}

#####
# setSytemId - 

sub setCharacterStream {
    my $self = shift;
    $self->{CHARACTERSTREAM} = shift;
}

sub getCharacterStream {
    my $self = shift;
    return $self->{CHARACTERSTREAM};
}

#####
# debug - cannonical debugging stuff from
# http://www.perl.com/CPAN-local/doc/manual/html/pod/perltoot/Debuging_Methods.html

sub debug {
    my $self = shift;
    warn "usage: thing->debug(level)"    unless @_ == 1;
    my $level = shift;
    if (ref($self))  {
	$self->{"_DEBUG"} = $level;         # just myself
    } else {
	$Debugging        = $level;         # whole class
    }
}

#####
# DESTROY - hook to display debugging info

sub DESTROY {
    my $self = shift;
    if ($Debugging || $self->{"_DEBUG"}) {
	warn "W3C::SAX::InputSource destroying $self " . $self->name;
    }
    -- ${ $self->{"_OUTSTANDING"} };
}

package W3C::SAX::FileInputSource;
@W3C::SAX::FileInputSource::ISA = ('W3C::SAX::InputSource');
use W3C::Util::Exception;

sub new {
    my ($proto, $source) = @_;
    my @lines;
    my $publicId;
    if ($source eq '-') {
	@lines = <STDIN>;
	$publicId = join ('', @lines);
	$publicId=~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
	$publicId = 'data:'.$publicId;
    } else {
	use W3C::Util::Exception;
	open(SOURCE, $source) || &throw(new W3C::Util::FileNotFoundException({-filename => $source}));
	@lines = <SOURCE>;
	close(SOURCE);
	if ($source !~ m/^\//) {
	    my $cwd = `pwd`;
	    chomp $cwd;
	    $source = $cwd.'/'.$source;
	    while ($source =~ s/\.\.\/[^\/]+\///) {}
	    while ($source =~ s/\/\.\//\//) {}
	}
	$publicId = 'file:'.$source;
    }

    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new(\@lines);
    $self->setPublicId($publicId);
    return $self;
}

#####
# END - hook to display debugging info

sub END {
    if ($Debugging) {
	print "All InputSources are going away now.\n";
    }
}

1;

package W3C::SAX::InputSource; # so make doesn't complain about the package

__END__

=head1 NAME

W3C::SAX::InputSource.pm - partial implementation of the Java SAX InputSource

=head1 SYNOPSIS

  use W3C::SAX::InputSource;
  my $inputSource = new W3C::SAX::FileInputSource($source);
  eval {$xmlParser->parse($inputSource);};

=head1 DESCRIPTION

  This is a partial implementation of the Java SAX InputSource. It stores the
  data stream in a scalar, emulating either a ByteStream or a CharacterStream,
  I'm not sure which.

=head2 SAX use:

  use W3C::SAX::XmlParser;
  use W3C::SAX::VerboseXmlHandler;
  use W3C::SAX::InputSource;
  use W3C::SAX::XmlElementTree;
  my $xmlParser = new W3C::SAX::PerlXmlParser;
  my $verboseHandler = new W3C::SAX::VerboseXmlHandler;
  my $treeHandler = new W3C::SAX::XmlElementTree;

  my $handlerList = new W3C::SAX::HandlerList([$treeHandler, $verboseHandler], [$verboseHandler]);

  # set the W3C::SAX::HandlerList be the document handler and the error handler
  $xmlParser->setDocumentHandler($handler);
  $xmlParser->setErrorHandler($handler);

  # assign a URI to the XML in InputSorce and parse it
  my $inputSource = new W3C::SAX::FileInputSource($source);
  eval {$xmlParser->parse($inputSource);};


This module is part of the W3C::SAX CPAN module.

=head1 AUTHOR

Eric Prud\'hommeaux <eric@w3.org>

=head1 SEE ALSO

perl(1).

=cut
