#Copyright Massachusetts Institute of technology, 1998.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

use strict;
require Exporter;
require AutoLoader;

$W3C::SAX::SAXException::REVISION = '$Id: SAXException.pm,v 1.12 1999/12/14 13:14:41 eric Exp $ ';

package W3C::SAX::SAXException;
use vars qw($VERSION $DSLI @ISA @EXPORT @EXPORT_OK);
@ISA = qw(W3C::Util::Exception Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = 0.91;
$DSLI = 'adpO';

#####
# per-object data
# -message	-  text of warning message
# -exception	-  complete type with whatever exception we write

#####
# new - aggregation of all constructors

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self;
    my $arg1 = shift;
    if (ref $arg1 eq '') {
	$self = $class->SUPER::new({-message => $arg1, -exception => shift});
    } elsif (ref $arg1 eq 'HASH') {
	$self = $class->SUPER::new($arg1);
    } else {
	$self = $class->SUPER::new({-exception => $arg1, -message => $arg1->toString});
    }
    # "private" data
    bless ($self, $class);
    $self->fillInStackTrace;
    return $self;
}

#####
# message - get/set message

sub messageNUKE {
    my $self = shift;
    if (@_) { $self->{-message} = shift }
    return $self->{-message};
}

#####
# exception - get/set exception

sub exceptionNUKE {
    my $self = shift;
    if (@_) { $self->{-exception} = shift }
    return $self->{-exception};
}

#####
# toString - get text format of this exception

sub toStringNUKE {
    my $self = shift;
    return $self->{-message} if (defined $self->{-message});
    return $self->{-exception}->toString() if (defined $self->{-exception});
    return undef;
}

sub getSpecificMessage {return $_[0]->{-message} ? $_[0]->{-message} : $_[0]->{-exception}->toString;}
sub getException {return $_[0]->{-exception}}

1;

__END__

=head1 NAME

W3C::SAX::SAXException - see SAX spec

=head1 SYNOPSIS

@@@

=head1 DESCRIPTION

This module is part of the W3C::SAX CPAN module.

=head1 AUTHOR

Eric Prud\'hommeaux <eric@w3.org>

=head1 SEE ALSO

W3C::SAX::XmlParser(3) W3C::SAX::SAXParseException(3) perl(1).

=cut
