#Copyright Massachusetts Institute of technology, 1998.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

use strict;
require Exporter;
require AutoLoader;

use W3C::SAX::SAXException;

$W3C::SAX::SAXParseException::REVISION = '$Id: SAXParseException.pm,v 1.12 1999/12/14 13:14:41 eric Exp $ ';

package W3C::SAX::SAXParseException;
use vars qw($VERSION $DSLI @ISA @EXPORT @EXPORT_OK);
@ISA = qw(W3C::SAX::SAXException Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = 0.91;
$DSLI = 'adpO';

#####
# per-object data
# -message	-  text of warning message
# -locator
# -publicid
# -systemid
# -linenumber
# -columnnumber
# -exception	-  complete type with whatever exception we write

#####
# new - aggregation of all constructors

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($message, $locator, $publicId, $systemId
	, $lineNumber, $columnNumber, $exception);
    $message = shift;
    my $arg2 = shift;
    if (ref $arg2 eq '') {
	$locator = undef;
	$publicId = $arg2;
	$systemId = shift;
	$lineNumber = shift;
	$columnNumber = shift;
    } else {
	$locator = $arg2;
	$publicId = $locator->getPublicId;
	$systemId = $locator->getSystemId;
	$lineNumber = $locator->getLineNumber;
	$columnNumber = $locator->getColumnNumber;
    }
    $exception = shift;
    my $self = $class->SUPER::new({-message => $message, -esception => $exception, -locator => $locator, 
				   -publicId => $publicId, -systemId => $systemId, 
				   -lineNumber => $lineNumber, -columnNumber => $columnNumber});
    bless ($self, $class);
    $self->fillInStackTrace;
    return $self;
}

#####
# publicId - get/set publicId

sub getPublicId {
    my $self = shift;
    return $self->{-publicId};
}

#####
# systemId - get/set systemId

sub getSystemId {
    my $self = shift;
    return $self->{-systemId};
}

#####
# lineNumber - get/set lineNumber

sub getLineNumber {
    my $self = shift;
    return $self->{-lineNumber};
}

#####
# columnNumber - get/set columnNumber

sub getColumnNumber {
    my $self = shift;
    return $self->{-columnNumber};
}

#####
# getMessage

sub getMessageNUKE {
    my $self = shift;
    return $self->{-message};
}

#####
# toString - get text format of this exception

sub toString {
    my $self = shift;
    my $prefix = $self->{-publicId}.':'.$self->{-systemId}.':'.
	$self->{-lineNumber}.':'.$self->{-columnNumber};
    $prefix .= '('.$self->{-locator}->getByteNumber.')' if ($self->{-locator}->can('getByteNumber'));
    return $prefix.':'.$self->{-message} if (defined $self->{-message});
    return .':'.$self->{-exception}->toString() if (defined $self->{-exception});
    return undef;
}

1;

__END__

=head1 NAME

W3C::SAX::SAXParseException - see SAX spec

=head1 SYNOPSIS

@@@

=head1 DESCRIPTION

This module is part of the W3C::SAX CPAN module.

=head1 AUTHOR

Eric Prud\'hommeaux <eric@w3.org>

=head1 SEE ALSO

W3C::SAX::XmlParser(3) W3C::SAX::SAXException(3) perl(1).

=cut
